# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Tests for NIH CXR14 dataset."""

import pytest


@pytest.mark.skip(reason="Test need to be updated")
def test_protocol_consistency():
    from ptbench.data.nih_cxr14_re import dataset

    # Default protocol
    subset = dataset.subsets("default")
    assert len(subset) == 3

    assert "train" in subset
    assert len(subset["train"]) == 98637
    for s in subset["train"]:
        assert s.key.startswith("images/000")

    assert "validation" in subset
    assert len(subset["validation"]) == 6350
    for s in subset["validation"]:
        assert s.key.startswith("images/000")

    assert "test" in subset
    assert len(subset["test"]) == 4054
    for s in subset["test"]:
        assert s.key.startswith("images/000")

    # Check labels
    for s in subset["train"]:
        for element in list(set(s.label)):
            assert element in [0.0, 1.0]

    for s in subset["validation"]:
        for element in list(set(s.label)):
            assert element in [0.0, 1.0]

    for s in subset["test"]:
        for element in list(set(s.label)):
            assert element in [0.0, 1.0]


@pytest.mark.skip(reason="Test need to be updated")
@pytest.mark.skip_if_rc_var_not_set("datadir.nih_cxr14_re")
def test_loading():
    from ptbench.data.nih_cxr14_re import dataset

    def _check_size(size):
        if size == (1024, 1024):
            return True
        return False

    def _check_sample(s):
        data = s.data
        assert isinstance(data, dict)
        assert len(data) == 2

        assert "data" in data
        assert _check_size(data["data"].size)  # Check size
        assert data["data"].mode == "RGB"  # Check colors

        assert "label" in data
        assert len(data["label"]) == 14  # Check labels

    limit = 30  # use this to limit testing to first images only, else None

    subset = dataset.subsets("default")
    for s in subset["train"][:limit]:
        _check_sample(s)
